--[[-------------------------------------------------------------------------
Copyright (c) 2010 Jan Erik Hanssen

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
---------------------------------------------------------------------------]]


--[[-------------------------------------------------------------------------
Configuration
---------------------------------------------------------------------------]]

local cfg = {
	-- geometry
	xpos = 0,
	ypos = 53,
	width = 37.9, -- width per combo point frame (5 in total)
	height = 5,

	-- anchor
	anchor = "CENTER",

	-- spacing between each combo point frame (4 in total)
	spacing = 2.7,

	-- frame padding
	padding = 2,

	-- fade
	fadein = 0.5,       -- fade delay for each combo point frame
	fadeout = 0.5,      -- fade delay for each combo point frame
	framefadein = 0.2,  -- fade delay for entering combat
	framefadeout = 0.2, -- fade delay for leaving combat

	-- colors
	comboColors = {
		{ r = 1, g = 0.86, b = 0.1, a = 1 },
		{ r = 1, g = 0.86, b = 0.1, a = 1 },
		{ r = 1, g = 0.86, b = 0.1, a = 1 },
		{ r = 1, g = 0.86, b = 0.1, a = 1 },
		{ r = 1, g = 0.86, b = 0.1, a = 1 },
	},
	bgColor = { r = 0.3, g = 0.3, b = 0.3, a = 0.7 },

	-- text
	showtext = true,
	font = "Fonts\\FRIZQT__.TTF",
	fontsize = 13,
	fontstyle = "OUTLINE",

	-- texture
	statusbar = "Interface\\TargetingFrame\\UI-StatusBar",

	-- hide when out of combat
	hideooc = true,

	-- Hide when no energy is available (i.e. druid out of cat form)
	hidenoenergy = true,
}

--[[-------------------------------------------------------------------------------
	Code
---------------------------------------------------------------------------------]]

local incombat  = false
local visible   = true
local powertype = nil
local unit      = "player"
local points    = 0
local locked    = true

local addon = CreateFrame("Frame", "naiCombo", UIParent)

-- the allmighty
function addon:new()
	local realwidth = (cfg.width * 5) + (cfg.spacing * 4)
	
	self:ClearAllPoints()
	self:SetWidth(realwidth)
	self:SetHeight(cfg.height)
	self:SetPoint(cfg.anchor, UIParent, cfg.anchor, cfg.xpos, cfg.ypos)
	self:SetMovable(true)
	
	self.bg = self:CreateTexture(nil, "BACKGROUND")
	self.bg:ClearAllPoints()
	--self.bg:SetAllPoints(self)
	self.bg:SetPoint("TOPLEFT", self, "TOPLEFT", -cfg.padding, cfg.padding)
	self.bg:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", cfg.padding, -cfg.padding)
	self.bg:SetTexture(cfg.statusbar)
	self.bg:SetVertexColor(cfg.bgColor.r, cfg.bgColor.g, cfg.bgColor.b)
	self.bg:SetAlpha(cfg.bgColor.a)
	
	self.combos = {}
	
	local cx = 0
	for i = 1, 5 do
		local combo = CreateFrame("Frame", nil, self)
		combo:ClearAllPoints()
		combo:SetPoint("TOPLEFT", self, "TOPLEFT", cx, 0)
		combo:SetPoint("BOTTOMRIGHT", self, "BOTTOMLEFT", cx + cfg.width, 0)
		--combo:SetFrameStrata("HIGH")
		combo.bg = combo:CreateTexture(nil, "BACKGROUND")
		combo.bg:ClearAllPoints()
		combo.bg:SetAllPoints(combo)
		combo.bg:SetTexture(cfg.statusbar)
		local curColor = cfg.comboColors[i]
		combo.bg:SetVertexColor(curColor.r, curColor.g, curColor.b)
		combo.bg:SetAlpha(curColor.a)
		--combo:Hide()
		combo:SetAlpha(0)
		self.combos[i] = combo
		
		cx = cx + cfg.width + cfg.spacing
	end

	self.overlay = CreateFrame("Frame", nil, self)
	self.overlay:ClearAllPoints()
	self.overlay:SetAllPoints(self)
	self.comboText = self.overlay:CreateFontString(nil, "OVERLAY")
	self.comboText:SetFont(cfg.font, cfg.fontsize, cfg.fontstyle)
	self.comboText:SetShadowOffset(1, -1)
	self.comboText:SetShadowColor(0, 0, 0, 1)
	self.comboText:SetJustifyH("CENTER")
	self.comboText:ClearAllPoints()
	self.comboText:SetAllPoints(self.overlay)

	self:RegisterEvent("PLAYER_LOGIN")
	self:RegisterEvent("UNIT_DISPLAYPOWER")
	self:RegisterEvent("UNIT_COMBO_POINTS")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	if cfg.hideooc then
		self:RegisterEvent("PLAYER_REGEN_DISABLED")
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
	end
	self:RegisterEvent("UNIT_ENTERED_VEHICLE")
	self:RegisterEvent("UNIT_EXITED_VEHICLE")

	self:SetScript("OnEvent", self.event)
end

function addon:toggleCombo()
	if not locked then
		return
	end
	if not cfg.hideooc and not cfg.hidenoenergy then
		return
	end

	if cfg.hidenoenergy then
		if powertype == SPELL_POWER_ENERGY then
			if cfg.hideooc then
				if not visible and points > 0 then
					UIFrameFadeIn(self, cfg.framefadein)
					visible = true
				end
			elseif not visible and points > 0 then
				UIFrameFadeIn(self, cfg.framefadein)
				visible = true
			end
		else
			if visible then
				UIFrameFadeOut(self, cfg.framefadeout)
				visible = false
			end
			return
		end
	end

	if cfg.hideooc then
		if not visible and points > 0 then
			UIFrameFadeIn(self, cfg.framefadein)
			visible = true
		elseif not incombat and points == 0 and visible then
			UIFrameFadeOut(self, cfg.framefadeout)
			visible = false
		end
	end
end

function addon:updateCombo()
	local pt = GetComboPoints(unit)
	if pt == points then
		self:toggleCombo()
		return
	end
	
	if pt > points then
		for i = points + 1, pt do
			UIFrameFadeIn(self.combos[i], cfg.fadein)
		end
	else
		for i = pt + 1, points do
			UIFrameFadeOut(self.combos[i], cfg.fadeout)
		end
	end
	
	if showtext then
		if pt > 0 then
			self.comboText:SetText(pt)
		else
			self.comboText:SetText("")
		end
	end
	
	points = pt
	
	self:toggleCombo()
end

function addon:event(event, ...)
	if event == "PLAYER_LOGIN" then
		if powertype ~= nil then return end

		powertype, _ = UnitPowerType("player")
		if UnitHasVehicleUI("player") then
			local powervehicle, _ = UnitPowerType("vehicle")
			if powervehicle == SPELL_POWER_ENERGY then
				unit = "vehicle"
				powertype = powervehicle
			end
		end

		if cfg.hideooc or (cfg.hidenoenergy and powertype ~= SPELL_POWER_ENERGY) then
			visible = false
			self:SetAlpha(0)
		end
	elseif event == "UNIT_COMBO_POINTS" then
		local curunit = select(1, ...)
		if curunit ~= unit then return end
		
		self:updateCombo()
	elseif event == "PLAYER_TARGET_CHANGED" then
		self:updateCombo()
	elseif event == "PLAYER_REGEN_DISABLED" then
		incombat = true

		self:toggleCombo()
	elseif event == "PLAYER_REGEN_ENABLED" then
		incombat = false

		self:toggleCombo()
	elseif event == "UNIT_DISPLAYPOWER" then
		local curunit = select(1, ...)
		if curunit ~= unit then return end
		
		powertype, _ = UnitPowerType(unit)

		self:toggleCombo()
	elseif event == "UNIT_ENTERED_VEHICLE" then
		local curunit = select(1, ...)
		if curunit ~= "player" then return end

		local powervehicle, _ = UnitPowerType("vehicle")
		if powervehicle == SPELL_POWER_ENERGY then
			unit = "vehicle"
			points = 0
			powertype = powervehicle

			self:toggleCombo()
		end
	elseif event == "UNIT_EXITED_VEHICLE" then
		local curunit = select(1, ...)
		if curunit ~= "player" then return end

		unit = "player"
		points = 0
		powertype, _ = UnitPowerType(unit)

		self:toggleCombo()
	end
end

function addon:reset()
	local realwidth = (cfg.width * 5) + (cfg.spacing * 4)

	self:SetUserPlaced(false)
	self:ClearAllPoints()
	self:SetWidth(realwidth)
	self:SetHeight(cfg.height)
	self:SetPoint(cfg.anchor, UIParent, cfg.anchor, cfg.xpos, cfg.ypos)
end

function addon:toggleLock()
	if locked then
		locked = false
		self:EnableMouse(true)
		self:RegisterForDrag("LeftButton")
		self:SetScript("OnDragStart", function(self) 
			self:StartMoving()
		end)
		self:SetScript("OnDragStop", function(self)
			self:StopMovingOrSizing()
		end)
		UIFrameFadeIn(self, cfg.framefadein)
		visible = true
	else
		locked = true
		self:EnableMouse(false)
		self:RegisterForDrag(nil)
		self:SetScript("OnDragStart", nil)
		self:SetScript("OnDragStop", nil)
		self:toggleCombo()
	end
end

SLASH_COMBOS1 = "/combos"
SlashCmdList["COMBOS"] = function(msg)
	if msg == "lock" then
		addon:toggleLock()
	elseif msg == "reset" then
		addon:reset()
	else
		DEFAULT_CHAT_FRAME:AddMessage("combos <lock> <reset>")
	end
end

-- and... go!
addon:new()